"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var EMPTY_CALL = function () { };
var Signal = (function () {
    function Signal() {
        this.isDispatching = false;
        this.detachAllFlag = false;
    }
    Signal.prototype.await = function () {
        var _this = this;
        return new Promise(function (resolve) { return _this.addOnce(resolve); });
    };
    Signal.prototype.add = function (callback, context, once) {
        var _this = this;
        var call = once ? function (param) {
            callback.call(context, param);
            _this.removeBinding(item);
        } : function (param) {
            callback.call(context, param);
        };
        var item = {
            active: true,
            call: call,
        };
        if (!this.isDispatching) {
            this.doAdd(item);
        }
        else {
            var addRequest = { binding: item };
            if (this.addRequests) {
                addRequest.next = this.addRequests;
            }
            this.addRequests = addRequest;
        }
        return {
            setActive: function (active) {
                item.active = active;
                if (active) {
                    item.call = call;
                }
                else {
                    item.call = EMPTY_CALL;
                }
            },
            isActive: function () { return item.active; },
            detach: function () {
                if (item.isDetached)
                    return;
                _this.removeBinding(item);
            },
        };
    };
    Signal.prototype.addOnce = function (callback, context) {
        return this.add(callback, context, true);
    };
    Signal.prototype.dispatch = function (param) {
        if (this.isDispatching) {
            throw new Error('Signal.dispatch() was called within the call stack of another dispatch.');
        }
        this.isDispatching = true;
        var current = this.head;
        while (current) {
            current.call(param);
            current = current.next;
        }
        this.isDispatching = false;
        if (this.detachAllFlag) {
            this.detachAllFlag = false;
            this.detachAll();
            this.detachRequests = undefined;
        }
        else {
            if (this.addRequests) {
                this.processAddRequests();
            }
            if (this.detachRequests) {
                this.processDetachRequests();
            }
        }
    };
    Signal.prototype.detachAll = function () {
        if (this.isDispatching) {
            this.detachAllFlag = true;
            return;
        }
        var current = this.head;
        while (current) {
            var temp = current;
            current = current.next;
            temp.call = undefined;
            temp.isDetached = true;
            temp.prev = temp.next = undefined;
        }
        this.head = this.tail = undefined;
    };
    Signal.prototype.doAdd = function (binding) {
        if (!this.tail) {
            this.head = this.tail = binding;
        }
        else {
            this.tail.next = binding;
            binding.prev = this.tail;
            this.tail = binding;
        }
    };
    Signal.prototype.processDetachRequests = function () {
        var current = this.detachRequests;
        while (current) {
            this.doRemoveBinding(current.binding);
            current = current.next;
        }
        this.detachRequests = undefined;
    };
    Signal.prototype.processAddRequests = function () {
        var current = this.addRequests;
        while (current) {
            this.doAdd(current.binding);
            current = current.next;
        }
        this.addRequests = undefined;
    };
    Signal.prototype.removeBinding = function (binding) {
        binding.isDetached = true;
        if (!this.isDispatching) {
            this.doRemoveBinding(binding);
            return;
        }
        var detachRequest = { binding: binding };
        if (this.detachRequests) {
            detachRequest.next = this.detachRequests;
        }
        this.detachRequests = detachRequest;
    };
    Signal.prototype.doRemoveBinding = function (binding) {
        if (!binding.prev) {
            if (!binding.next) {
                this.head = this.tail = undefined;
            }
            else {
                this.head = binding.next;
                this.head.prev = undefined;
            }
        }
        else {
            if (!binding.next) {
                this.tail = binding.prev;
                this.tail.next = undefined;
            }
            else {
                binding.prev.next = binding.next;
                binding.next.prev = binding.prev;
            }
        }
        binding.prev = binding.next = binding.call = undefined;
    };
    return Signal;
}());
exports.Signal = Signal;
exports.signalGroup = function () {
    var disposers = [];
    return {
        add: function (signal, callback) {
            disposers.push(signal.add(function (param) {
                callback(param);
            }));
        },
        dispose: function () {
            disposers.forEach(function (d) { return d.detach(); });
        },
        setActive: function (active) {
            disposers.forEach(function (d) { return d.setActive(active); });
        },
    };
};
//# sourceMappingURL=signal.js.map