"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const EMPTY_CALL = () => { };
class Signal {
    constructor() {
        this.isDispatching = false;
        this.detachAllFlag = false;
    }
    await() {
        return new Promise(resolve => this.addOnce(resolve));
    }
    add(callback, context, once) {
        const call = once ? (param) => {
            callback.call(context, param);
            this.removeBinding(item);
        } : (param) => {
            callback.call(context, param);
        };
        let item = {
            active: true,
            call,
        };
        if (!this.isDispatching) {
            this.doAdd(item);
        }
        else {
            const addRequest = { binding: item };
            if (this.addRequests) {
                addRequest.next = this.addRequests;
            }
            this.addRequests = addRequest;
        }
        return {
            setActive: (active) => {
                item.active = active;
                if (active) {
                    item.call = call;
                }
                else {
                    item.call = EMPTY_CALL;
                }
            },
            isActive: () => item.active,
            detach: () => {
                if (item.isDetached)
                    return;
                this.removeBinding(item);
            },
        };
    }
    addOnce(callback, context) {
        return this.add(callback, context, true);
    }
    dispatch(param) {
        if (this.isDispatching) {
            throw new Error('Signal.dispatch() was called within the call stack of another dispatch.');
        }
        this.isDispatching = true;
        let current = this.head;
        while (current) {
            current.call(param);
            current = current.next;
        }
        this.isDispatching = false;
        if (this.detachAllFlag) {
            this.detachAllFlag = false;
            this.detachAll();
            this.detachRequests = undefined;
        }
        else {
            if (this.addRequests) {
                this.processAddRequests();
            }
            if (this.detachRequests) {
                this.processDetachRequests();
            }
        }
    }
    detachAll() {
        if (this.isDispatching) {
            this.detachAllFlag = true;
            return;
        }
        let current = this.head;
        while (current) {
            const temp = current;
            current = current.next;
            temp.call = undefined;
            temp.isDetached = true;
            temp.prev = temp.next = undefined;
        }
        this.head = this.tail = undefined;
    }
    doAdd(binding) {
        if (!this.tail) {
            this.head = this.tail = binding;
        }
        else {
            this.tail.next = binding;
            binding.prev = this.tail;
            this.tail = binding;
        }
    }
    processDetachRequests() {
        let current = this.detachRequests;
        while (current) {
            this.doRemoveBinding(current.binding);
            current = current.next;
        }
        this.detachRequests = undefined;
    }
    processAddRequests() {
        let current = this.addRequests;
        while (current) {
            this.doAdd(current.binding);
            current = current.next;
        }
        this.addRequests = undefined;
    }
    removeBinding(binding) {
        binding.isDetached = true;
        if (!this.isDispatching) {
            this.doRemoveBinding(binding);
            return;
        }
        const detachRequest = { binding };
        if (this.detachRequests) {
            detachRequest.next = this.detachRequests;
        }
        this.detachRequests = detachRequest;
    }
    doRemoveBinding(binding) {
        if (!binding.prev) {
            if (!binding.next) {
                this.head = this.tail = undefined;
            }
            else {
                this.head = binding.next;
                this.head.prev = undefined;
            }
        }
        else {
            if (!binding.next) {
                this.tail = binding.prev;
                this.tail.next = undefined;
            }
            else {
                binding.prev.next = binding.next;
                binding.next.prev = binding.prev;
            }
        }
        binding.prev = binding.next = binding.call = undefined;
    }
}
exports.Signal = Signal;
exports.signalGroup = () => {
    const disposers = [];
    return {
        add: (signal, callback) => {
            disposers.push(signal.add(param => {
                callback(param);
            }));
        },
        dispose: () => {
            disposers.forEach(d => d.detach());
        },
        setActive: (active) => {
            disposers.forEach(d => d.setActive(active));
        },
    };
};
//# sourceMappingURL=signal.js.map